
package polyglot.ext.ibex.ast;

import polyglot.ast.*;
import polyglot.ext.jl.ast.*;
import polyglot.types.*;
import polyglot.ext.ibex.types.*;
import polyglot.ext.ibex.visit.*;
import polyglot.util.*;
import polyglot.visit.*;
import java.util.*;

/**
 * Node representing a semantic action.
 */
public class Tagged_c extends RhsElement_c implements Tagged {
    SymbolNode sym;
    String tag;

    public Tagged_c(Position pos, String tag, SymbolNode sym) {
        super(pos);
        this.sym = sym;
        this.tag = tag;
    }

    public String tag() { return tag; }
    public Tagged tag(String tag) {
        Tagged_c n = (Tagged_c) copy();
        n.tag = tag;
        return n;
    }

    public SymbolNode symbol() { return sym; }
    public Tagged symbol(SymbolNode symbol) {
        Tagged_c n = (Tagged_c) copy();
        n.sym = sym;
        return n;
    }

    public void addDecls(Context c) {
        IbexTypeSystem ts = (IbexTypeSystem) c.typeSystem();
        if (sym.symbolInstance() == null) {
            return;
        }
        c.addVariable(ts.localInstance(position(), Flags.FINAL,
                                       sym.symbolInstance().symbolType(), tag));
    }

    protected Tagged_c reconstruct(SymbolNode sym) {
        if (sym != this.sym) {
            Tagged_c n = (Tagged_c) copy();
            n.sym = sym;
            return n;
        }
        return this;
    }

    public Node visitChildren(NodeVisitor v) {
        SymbolNode sym = (SymbolNode) visitChild(this.sym, v);
        return reconstruct(sym);
    }   

    public Node typeCheck(TypeChecker tc) throws SemanticException {
        TypeSystem ts = tc.typeSystem();

        if (symbolInstance().symbolType().isVoid()) {
            throw new SemanticException("Cannot have an action parameter of type void.", position());
        }

        return this;
    }

    public Symbol symbolInstance() {
        return sym.symbolInstance();
    }

    public Node rewrite(Rewriter rw) {
        return this;
    }
}
