package polyglot.ext.ibex.types;

import polyglot.types.*;
import polyglot.ext.jl.types.*;
import polyglot.util.*;
import java.util.*;

public class RhsCase_c extends TypeObject_c implements RhsCase {
    List elements;
    List tags;

    public RhsCase_c(IbexTypeSystem ts, Position pos, List elements, List tags) {
        super(ts, pos);
        this.elements = elements;
        this.tags = tags;
    }

    public List elements() {
        return elements;
    }

    public List tags() {
        return tags;
    }

    public boolean isCanonical() {
        return true;
    }

    public boolean matches(Rhs rhs) {
        if (rhs instanceof RhsCase) {
            return hasSameElements((RhsCase) rhs);
        }
        return false;
    }

    public boolean hasSameElements(RhsCase rhs) {
        if (elements.size() != rhs.elements().size()) {
            return false;
        }

        Iterator i1 = elements().iterator();
        Iterator i2 = rhs.elements().iterator();
        while (i1.hasNext() && i2.hasNext()) {
            Symbol s1 = (Symbol) i1.next();
            Symbol s2 = (Symbol) i2.next();
            if (! s1.equals(s2)) {
                return false;
            }
        }

        return true;
    }

    public String actionMethodName(Nonterminal lhs) {
        String methodName = "action$" + lhs.name();
        for (Iterator j = elements().iterator(); j.hasNext(); ) {
            Symbol s = (Symbol) j.next();
            methodName += "$" + s.name();             
        }
        return methodName;
    }

    public List actionMethodFormalTypes() {
        List l = new ArrayList(elements().size());
        Iterator i = elements().iterator();
        Iterator j = tags().iterator();

        while (i.hasNext() && j.hasNext()) {
            Symbol s = (Symbol) i.next();
            String tag = (String) j.next();
            if (tag != null) {
                l.add(s.symbolType());
            }
        }
        return l;
    }

    public String toString() {
        String str = "";

        Iterator i = elements().iterator();
        Iterator j = tags().iterator();

        while (i.hasNext() && j.hasNext()) {
            Symbol s = (Symbol) i.next();
            String tag = (String) j.next();

            str += s;

            if (tag != null) {
                str += ":" + tag;
            }

            if (i.hasNext()) {
                str += " ";
            }
        }

        return str;
    }
}
