package polyglot.ext.j0.ast;

import polyglot.ast.*;

import polyglot.ext.jl.ast.*;
import polyglot.types.*;
import polyglot.util.*;
import polyglot.visit.*;
import polyglot.main.Options;
import polyglot.util.StringUtil;
import polyglot.ext.j0.types.*;

import java.util.*;

/**
 * An <code>Import</code> is an immutable representation of a Java
 * <code>import</code> statement.  It consists of the string representing the
 * item being imported and the kind which is either indicating that a class
 * is being imported, or that an entire package is being imported.
 */
public class ImportStaticMethod_c extends Import_c implements ImportStaticMethod  {

    List parameters;

    public List parameterTypes () {
	ListIterator paramIterator = parameters.listIterator(); 
	LinkedList typeList = new LinkedList();
	while(paramIterator.hasNext()) {
	    Formal_c param = (Formal_c)paramIterator.next();
	    Typed tn = (Typed)param.type();
	    typeList.add(tn.type());
	}

	return typeList;
	
    }

    public ImportStaticMethod_c(Position pos, String name, List parameters) {
	super(pos, STATIC_METHOD, name);
	this.parameters = parameters;

	ListIterator paramIterator = parameters.listIterator(); 

    }
 
    /** Build type objects for the import. */
    public Node buildTypes(TypeBuilder tb) throws SemanticException {
        ImportTable it = tb.importTable();
	
	it.addClassImport(StringUtil.getPackageComponent(name));
	
        return this;
        
     }
 
    /** Check that this method exists */
    public Node typeCheck(TypeChecker tc) throws SemanticException {

	J0TypeSystem ts = (J0TypeSystem)tc.typeSystem();

 
        // The type must exist.
        String className = StringUtil.getPackageComponent(name);
	String methodName = StringUtil.getShortNameComponent(name);
	
	ClassType nt = (ClassType)ts.typeForName(className);

	if(ts.hasMethodNamed(nt, methodName)) {

	    List parameterTypes = parameterTypes();
	    MethodInstance m = ts.findMethod(nt, methodName, parameterTypes,nt);
	    // This last nt might be bogus..

	    J0MethodInstance jm = (J0MethodInstance)m.copy();

	    jm.setImportedStatic(name);

	    ts.addStaticMethod(name, methodName, parameterTypes, jm);

	} else {
	    throw new SemanticException("Cannot import " + name);
	}

	// add this methodinstance into the current context

        return this;
    }
 
    public String toString() {
	// TODO: have it print out the parameter types
        return "import " + name +  "()";
	//return "";
	
    }
 
    /** Write the import to an output file. */
    public void prettyPrint(CodeWriter w, PrettyPrinter tr) {

	if (! Options.global.fully_qualified_names) {
            w.write("//import ");
            w.write(name);
 
            w.write("()");
             
            w.write(";");
            w.newline(0);
        }
    }

    public Node visitChildren(NodeVisitor v) {
	parameters = visitList(parameters, v);
	return this;
    }

}
