package polyglot.ext.polyj.ast;

import polyglot.ast.*;
import polyglot.types.*;
import polyglot.util.*;
import polyglot.visit.*;

/**
 * An <code>AmbInstOrAccess</code> is an ambiguous piece of syntax
 * that represents either an instantiated parametric type
 * or an array access.  The parser has been unable
 * to distinguish which it is.  So a bit of semantic analysis (during
 * the disambiguation pass of the compiler) is necessary
 * to determine what the node should actually be.
 * 
 * For example, in A[B].foo(), A[B] could be either a type or value.  
 * In fact, A[B] must occur in the context of a Receiver in order
 * for the parser to be forced to make the node Ambiguous.  
 * That's why this node extends Receiver.
 * 
 * @see polyglot.ext.polyj.parse.InstOrAccess
 */
public interface AmbInstOrAccess extends Receiver, Ambiguous
{
    /**
     * The prefix of the node.  In A[B], this would be A.
     */
    Receiver prefix();
    AmbInstOrAccess prefix(Receiver prefix);    

    /**
     * The parameter of the node.  In A[B], this would be B.
     */
    Receiver param();
    AmbInstOrAccess param(Receiver param);
}
