package polyglot.ext.polyj.ast;

import polyglot.ext.polyj.types.*;
import polyglot.ext.polyj.util.*;
import polyglot.ext.polyj.visit.*;

import polyglot.ext.jl.ast.*;

import polyglot.ast.*;
import polyglot.types.*;
import polyglot.util.*;
import polyglot.visit.*;

import java.util.*;

/**
 * PolyJ translates NewArrays as method calls when the ultimate
 * base type of the array is a type parameter.
 */
public class PolyJNewArray_c extends NewArray_c implements PolyJNewArray
{
    /**
     * Creates a new <code>NewArray</code>.
     *
     * @param init The initializer expression. This argument may be
     *  <code>null</code> if there is not such expression.
     * @pre <code>addDims</code> >= 0 and each element of <code>dims</code>
     *  is of type <code>Expression</code>.
     */
    public PolyJNewArray_c( Position pos, TypeNode baseType, List dims, 
        int addDims, ArrayInit init)
    {
        super(pos, baseType, dims, addDims, init);
    }

    public void translate(CodeWriter w, Translator tr) {
        PolyJTypeSystem ts = (PolyJTypeSystem) tr.typeSystem();
	PolyJTranslator pjtr = (PolyJTranslator) tr;
        Context c = tr.context();
        TranslationTable tt
            = ((PolyJTypeSystem)c.typeSystem()).translationTable();
        ArrayType myType = type.toArray();
        Type elementType = myType.ultimateBase();

        if (elementType instanceof ParamType) {
            ParamType param = (ParamType) elementType;

            ClassType scopedClass = c.currentClass();
            NewArrayBinding binding = ts.NewArrayBinding(param, myType.dims(),
                scopedClass);

            w.write("((" 
	        + PolyJTranslator.adapterInterfaceFullName(scopedClass) 
		+ ")" + pjtr.whereObjExpr() + ")." + binding.name() + "(");
            w.begin(0);
            for(Iterator iter = dims().iterator(); iter.hasNext(); ) {
                w.begin(4);
                Expr e = (Expr) iter.next();
                e.del().translate(w, tr);
                if (iter.hasNext()){
                    w.write(",");
                    w.allowBreak(0);
                }
                w.end();
            }
            w.write(")");
            w.end();
        } else {
            super.translate(w, tr);
        }
    }

}


