package polyglot.ext.polyj.parse;

import polyglot.ext.polyj.ast.*;

import polyglot.ast.*;
import polyglot.visit.*;
import polyglot.types.*;
import polyglot.util.*;

import java.util.*;

/**
 * A <code>Sub</code> is a piece of syntax in a list inside of square brackets. 
 * It is either a parameter in the instantiation of a parameterized type
 * or an index in an array access.
 * 
 *    e.g., in:
 *      array[e], list[T], map[K, V]
 *    e, T, K, and V are initially represented in the AST by Subs.
 * 
 * No Sub node should ever make its way into the AST.
 */
public interface Sub {
    
    /** 
     * Prepend a Parse node to this Sub,
     * creating "base[this]". Zero or 2+ Subs inside the 
     * brackets is unambiguous and thus should be handled 
     * in the Parse hierarchy.
     */
    public Parse prepend(Parse base) throws Exception;
    
    /**
     * Convert this into an Expr, or die if that isn't possible.
     */
    public Expr toExpr() throws Exception;
    
    /**
     * Convert this into a TypeNode, or die if that isn't possible.
     */
    public TypeNode toType() throws Exception;

    /**
     * Convert this into a ParamNode, or die if it's ambiguous what
     * type of ParamNode to create.
     */
    public ParamNode toParam() throws Exception;
}
