package polyglot.ext.polyj.parse;

import polyglot.ast.*;
import polyglot.visit.*;
import polyglot.types.*;
import polyglot.util.*;

import polyglot.ext.jl.ast.*;

import java.util.*;  

/** 
 * A <code>Wrapper</code> wraps a <code>Parse</code> inside an AST node so that
 * it can be insert in the AST.  However, the tree <em>must</em> be visited
 * by an <code>UnwrapVisitor</code> before it is returned from the parser, otherwise
 * the other phases of the compiler will be unable to work with the tree!
 */
public class Wrapper extends Expr_c {
    Parse parse;

    public Wrapper(Parse parse) {
        super(parse.pos);
        this.parse = parse;
    }

    public Parse parse() {
        return parse;
    }
    
    public Node visitChildren(NodeVisitor v) {
        throw new InternalCompilerError("Cannot visit a Wrapper.");
    }

    public Node typeCheck(TypeChecker tc) throws SemanticException {
        throw new InternalCompilerError("Cannot type check a Wrapper");
    }

    public void translate(CodeWriter w, Translator tr) {
        throw new InternalCompilerError("Cannot translate a Wrapper");
    }

    public void dump(CodeWriter w) {
        w.write("POLYJ-WRAPPED-PARSE");
        super.dump(w);
    }
    /**
     * Visit this term in evaluation order.
     */
    public List acceptCFG(CFGBuilder v, List succs) {
        return succs;
    }
    
    public Term entry() {
      return this;
    }
}
