package polyglot.ext.polyj.util;

import java.util.*;
import polyglot.types.Package;
import polyglot.types.*;
import polyglot.util.*;
import polyglot.ext.polyj.types.*;
import polyglot.ext.polyj.visit.*;

public class InstantiationBinding_c implements InstantiationBinding {
    protected ClassType type;
    protected String adapterPackageName;
    protected String typePackageName;
    protected boolean needsAdapter;
    protected PolyJTypeSystem ts;

    public InstantiationBinding_c(ClassType type) {
        this.type = type;
        ts = (PolyJTypeSystem) type.typeSystem();
        adapterPackageName = adapterPackage();
        typePackageName = "";
        Package typePackage = type.package_();
        if (typePackage != null) {
            typePackageName = typePackage.fullName();
            if (!adapterPackageName.equals("")) {
                adapterPackageName += ".";
            } 
            adapterPackageName += typePackageName;
        }
        this.needsAdapter = doesNeedAdapter();
    }

    public ClassType type() { return type; }

    public boolean needsAdapterClass() {
        return needsAdapter;
    }

    public String getAdapterClassPackageName() {
        return adapterPackageName;
    }

    public String getAdapterClassFullName() {
        return adapterPackageName + "." + getAdapterClassShortName();
    }

    public MethodInstance getImplementingMethod(WhereBinding constraint) {
        return null;
    }

    public String getAdapterClassShortName() {
        String polyName = type.toString();
        if (!polyName.startsWith(typePackageName)) {
            throw new InternalCompilerError("Bad adapter class name");
        }
        int startAt = 0;
        int packLength = typePackageName.length();
        if (packLength != 0) {
            // skip over the dot at the end of the package name
            startAt = packLength + 1;
        }
        polyName = polyName.substring(startAt);
        return PolyJTranslator.mangle(polyName);
    }

    protected boolean doesNeedAdapter() {
        List args = ts.allTypeArgs(type);
        if (args.size() == 0) {
            return false;
        }

        for (Iterator i = args.iterator(); i.hasNext(); ) {
            if (i.next() instanceof ParamType) {
                return false;
            }
        }
        return true;
    }

    public String toString() {
        if (needsAdapterClass()) {
            return getAdapterClassFullName();
        } else {
            return "[none]";
        }
    }
    
    protected Flags meet(Flags f1, Flags f2) {
        if (f1.isPrivate() || f2.isPrivate()) {
            return Flags.PRIVATE;
        } else  if (f1.isProtected() || f2.isProtected()) {
            return Flags.PROTECTED;
        } else if (f1.isPublic() && f2.isPublic()) {
            return Flags.PUBLIC;
        } else {
            return Flags.NONE;
        }
    }
    
    protected String adapterPackage() {
        List args = ts.allTypeArgs(type);
        Flags strongestProt = Flags.PUBLIC;
        for (Iterator i = args.iterator(); i.hasNext(); ) {
            Type t = (Type) i.next();
            if (t.isClass()) {
                strongestProt = meet(strongestProt, t.toClass().flags());
            }
        }
        if (strongestProt == Flags.PRIVATE || strongestProt == Flags.PROTECTED) 
        {
            throw new InternalCompilerError("Illegal instantiation");
        } else if (strongestProt == Flags.NONE) {
            return ""; // goes in the same package as the type
        } else { // PUBLIC
            //return "";
            // If the type's package is sealed, we need to move the
            // instantiation to another package.  But that causes a bug
            // with the call to $clinit if the type isn't public.
            return "polyj.instantiations";
        }
    }
}
